from omforme.omforme import Omforme


def test_omforme():
    gen = (x for x in ('a', 'b', 'c', 'c', 'c', 'd', '1', '2', '3', 'e'))
    playbook = (
        ('b', 'ignore', lambda: None),
        ('d', 'collect', list()),
        ('e', 'return', lambda: None),
    )
    transform = Omforme(playbook)(gen)
    assert transform[1] == ('d', 'collect', ['1', '2', '3'])



"""CLI interface for single view visualization mapping parameter values to an area scale resembling a (quality) pie."""
import argparse
import sys

import omforme.omforme as api
from omforme import APP_ALIAS, APP_NAME


def parse_request(argv: list[str]) -> int | argparse.Namespace:
    """DRY."""
    parser = argparse.ArgumentParser(
        prog=APP_ALIAS, description=APP_NAME, formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        '--quiet',
        '-q',
        dest='quiet',
        default=False,
        action='store_true',
        help='work as quiet as possible (default: False)',
    )
    parser.add_argument(
        '--verbose',
        '-v',
        dest='verbose',
        default=False,
        action='store_true',
        help='work logging more information along the way (default: False)',
    )
    if not argv:
        parser.print_help()
        return 0

    options = parser.parse_args(argv)

    if options.verbose and options.quiet:
        parser.error('you cannot be quiet and verbose at the same time')

    return options


def main(argv: list[str] | None = None) -> int:
    """Delegate processing to functional module."""
    argv = sys.argv[1:] if argv is None else argv
    options = parse_request(argv)
    if isinstance(options, int):
        return 0
    return api.main(options)



import pathlib

import pytest

import omforme.cli as cli

TEST_PREFIX = pathlib.Path('test', 'fixtures')
DEFAULT_DOCUMENTS_PATH = TEST_PREFIX
TEST_MAKE_MISSING = 'missing-this-file-for-'


def test_parse_request(capsys):
    options = cli.parse_request([])
    assert options == 0  # type: ignore
    out, err = capsys.readouterr()
    assert 'usage: omforme [-h] [--quiet] [--verbose]' in out
    assert not err


def test_parse_request_pos_doc_root_not_present(capsys):
    with pytest.raises(SystemExit) as err:
        cli.parse_request([f'{TEST_MAKE_MISSING}{DEFAULT_DOCUMENTS_PATH}', '-q', '-v'])
    assert err.value.code == 2
    out, err = capsys.readouterr()
    assert not out
    message_part = f'omforme: error: unrecognized arguments: {TEST_MAKE_MISSING}{DEFAULT_DOCUMENTS_PATH}'
    assert message_part in err



"""Provide the class Omforme for transformations on a generator.

Use case example:

from omforme.omforme import Omforme

Given a stream of lines e.g. in gen

gen = (x for x in ('a', 'b', 'c', 'c', 'c', 'd', '1', '2', '3', 'e'))

with defining a playbook of:

playbook = (('b', 'ignore', lambda: None), ('d', 'collect', list()), ('e', 'return', lambda: None),)

when calling

transform = Omforme(playbook)(gen)

then transform should become:

(
    ('b', 'ignore', <function <lambda> at 0x...>),
    ('d', 'collect', ['1', '2', '3']),
    ('e', 'return', <function <lambda> at 0x...>)
)

"""
import argparse
from typing import no_type_check

from omforme import DEBUG, ENCODING, ENCODING_ERRORS_POLICY, log


class Omforme:
    """Provide a generator borrowing consumer that returns the transform applying playbook to the stream.

    The playbook is a tuple of (trigger, action, function) triplets.

    Actions are epected to be within ('ignore', 'collect', 'return')

    Example:

    >>> # from omforme.omforme import Omforme
    >>> gen = (x for x in ('a', 'b', 'c', 'c', 'c', 'd', '1', '2', '3', 'e'))
    >>> playbook = (('b', 'ignore', lambda: None), ('d', 'collect', list()), ('e', 'return', lambda: None),)
    >>> transform = Omforme(playbook)(gen) # doctest: +ELLIPSIS
    0 3 a b ignore <function <lambda> at 0x...> d
    0 3 b b ignore <function <lambda> at 0x...> d
    0 3 c b ignore <function <lambda> at 0x...> d
    0 3 c b ignore <function <lambda> at 0x...> d
    0 3 c b ignore <function <lambda> at 0x...> d
    1 3 1 d collect ['1'] e
    1 3 2 d collect ['1', '2'] e
    1 3 3 d collect ['1', '2', '3'] e
    1 3 e d collect ['1', '2', '3'] e
    >>> transform[1]
    ('d', 'collect', ['1', '2', '3'])
    >>>

    """

    @no_type_check
    def __init__(self, playbook):
        """Later alligator."""
        self.playbook = playbook

    @no_type_check
    def __call__(self, gen):
        """Apply transform to generator stream of events."""
        phase = 0
        trigger, what, where_to = self.playbook[phase]
        peek = self.playbook[phase + 1][0]
        stop = len(self.playbook)
        for data in gen:
            if data == peek:
                if phase + 2 < stop:
                    phase += 1
                    trigger, what, where_to = self.playbook[phase]
                    peek = self.playbook[phase + 1][0]
                    continue
            if what == 'return':
                break
            if what == 'collect' and data != peek:
                where_to.append(data)
            print(phase, stop, data, trigger, what, where_to, peek)

        return self.playbook


def main(options: argparse.Namespace) -> int:
    log.info(f'{DEBUG=}, {ENCODING=}, {ENCODING_ERRORS_POLICY=}')
    return 0


if __name__ == '__main__':
    import doctest

    doctest.testmod(verbose=True, optionflags=doctest.ELLIPSIS)
